#include <stdio.h>
#include <stdlib.h>

#define BUFFER_SIZE 5  // リングバッファのサイズ

struct RingBuffer {
    float buffer[BUFFER_SIZE]; // バッファ
    int head; // 書き込み位置
    int tail; // 読み出し位置
    int full; // バッファが満杯かどうか
    int empty; // バッファが空かどうか
};

// リングバッファの内容を表示
void print_RingBuffer(struct RingBuffer *q, char *msg, float val) {
    printf("%s(%1.1f): ", msg, val);
    for (int i = 0; i < BUFFER_SIZE; i++) {
        if (i == q->tail) {printf("tail=> "); }
        if (i == q->head) {printf("<=head "); }// ヘッドとテールの位置を表示
        printf("%1.1f", q->buffer[i]);
        printf(" ");
    }
    printf("\n");
}

// リングバッファの初期化
void init_RingBuffer(struct RingBuffer *q) {
    for (int i = 0; i < BUFFER_SIZE; i++) {
        q->buffer[i] = 0.0;
    }
    q->head = 0;
    q->tail = 0;
    print_RingBuffer(q, "init", 0.0);// バッファの初期化を表示
}

// キューにデータを追加（enqueue）
void enq(struct RingBuffer *q, float val) {
    q->buffer[q->head] = val;
    q->head = (q->head + 1) % BUFFER_SIZE;
    if (q->full) { // 溢れたら古いデータを削除
        q->tail = (q->tail + 1) % BUFFER_SIZE;
        q->full = 0;
    }
    q->full = (q->head == q->tail);// バッファが満杯かどうか
    print_RingBuffer(q, "enq", val); // データを表示
}

// キューからデータを取り出す（dequeue）
float deq(struct RingBuffer *q) {
    float val = q->buffer[q->tail]; // データを取り出す
    q->tail = (q->tail + 1) % BUFFER_SIZE; // 読み出し位置を進める
    q->full = 0; // バッファは空に
    print_RingBuffer(q, "deq", val); // データを表示
    return val;// データを返す
}

// リングバッファのテスト
int main(int argc, char *argv[]) {
    struct RingBuffer q = {0}; // リングバッファの初期化
    init_RingBuffer(&q);
    enq(&q, 11.0);
    enq(&q, 22.0);
    enq(&q, 33.0);
    enq(&q, 44.0);
    enq(&q, 55.0);
    float val1 = deq(&q); // 11.0
    float val2 = deq(&q); // 22.0
    float val3 = deq(&q); // 33.0
    float val4 = deq(&q); // 44.0
    float val5 = deq(&q); // 55.0
}
