#include <SDL.h>
#include <stdlib.h>

#define WINDOW_WIDTH  800 // 画面の幅
#define WINDOW_HEIGHT 600 // 画面の高さ
#define HISTORY_MAX   200 // 履歴の最大数

int main(int argc, char* argv[]) {
    SDL_Init(SDL_INIT_VIDEO); // SDLの初期化
    SDL_Window* window = SDL_CreateWindow(argv[0],
        SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
        WINDOW_WIDTH, WINDOW_HEIGHT, SDL_WINDOW_SHOWN
    );
    SDL_Renderer* renderer = SDL_CreateRenderer(window, -1, 2);

    // 軌跡の座標を記録する配列（リングバッファなし）
    int X[HISTORY_MAX];
    int Y[HISTORY_MAX];
    int x = WINDOW_WIDTH / 2;  // 初期位置（画面の中央）
    int y = WINDOW_HEIGHT / 2; // 初期位置（画面の中央）
    X[0] = x; // 初期位置を記録
    Y[0] = y; // 初期位置を記録

    for (int count = 1; count <= HISTORY_MAX; count++) {
        // イベント処理
        SDL_Event event;
        int quit = 0;
        while (SDL_PollEvent(&event)) {
            if (event.type == SDL_QUIT) { quit = 1; }
        }
        if (quit) { break; }

        // 軌跡の数が最大数に達したら古いものを消す
        if (count == HISTORY_MAX) {
            X[0] = X[count - 1];
            Y[0] = Y[count - 1];
            count = 1;
        }

        // ランダムに移動方向を決定（課題：ここを書き換えてみよう）
        int dx = 0;
        int dy = 0;
        int r = rand() % 4; // 0,1,2,3の乱数
        if (r == 0)      { dx =  1; } // 右→
        else if (r == 1) { dy =  1; } // 下↓
        else if (r == 2) { dx = -1; } // 左←
        else if (r == 3) { dy = -1; } // 上↑
        // 10ドット間隔で移動
        x += dx * 10;
        y += dy * 10;

        // 軌跡の履歴を追加
        X[count] = x;
        Y[count] = y;

        // 背景を描画してクリア
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);

        // 足跡の線を描画
        for (int i = 1; i <= count; i++) {
            int alpha = (128*i/count); // 透明度を変化させる
            SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_BLEND);
            SDL_SetRenderDrawColor(renderer, 255, 255, 255, alpha);
            SDL_RenderDrawLine(renderer, X[i-1], Y[i-1], X[i], Y[i]);
        }

        // 画面を更新
        SDL_RenderPresent(renderer);
        SDL_Delay(16); // 16ミリ秒待機 (約60fps)
    }

    // 終了
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();
}
