#include <SDL.h>
#include <stdlib.h>

#define WINDOW_WIDTH  800  // 画面の幅
#define WINDOW_HEIGHT 600  // 画面の高さ
#define HISTORY_MAX   200  // 履歴の最大数

int main(int argc, char* argv[]) {
    SDL_Init(SDL_INIT_VIDEO); // SDLの初期化
    SDL_Window* window = SDL_CreateWindow(argv[0],
        SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
        WINDOW_WIDTH, WINDOW_HEIGHT, SDL_WINDOW_SHOWN
    );
    SDL_Renderer* renderer = SDL_CreateRenderer(window, -1, 2);

    // 軌跡の座標を記録する配列（リングバッファあり）
    int X[HISTORY_MAX];
    int Y[HISTORY_MAX];
    int head = 0; // リングバッファの先頭位置
    int count = 0; // 現在の履歴数

    int x = WINDOW_WIDTH / 2;  // 初期位置（画面の中央）
    int y = WINDOW_HEIGHT / 2; // 初期位置（画面の中央）
    X[head] = x; // 初期位置を記録
    Y[head] = y; // 初期位置を記録
    count++;

    for (;;) {
        // イベント処理
        SDL_Event event;
        int quit = 0;
        while (SDL_PollEvent(&event)) {
            if (event.type == SDL_QUIT) {
                quit = 1;
            }
        }
        if (quit) { break; }

        // ランダムに移動方向を決定（課題：ここを書き換えてみよう）
        int dx = 0, dy = 0;
        int r = rand() % 4; // 0,1,2,3の乱数
        if (r == 0)      { dx =  1; } // 右→
        else if (r == 1) { dy =  1; } // 下↓
        else if (r == 2) { dx = -1; } // 左←
        else if (r == 3) { dy = -1; } // 上↑
        // 10ドット間隔で移動
        x += dx * 10;
        y += dy * 10;

        // 軌跡の履歴をリングバッファに追加
        head = (head + 1) % HISTORY_MAX; // 次の位置に進める
        X[head] = x;
        Y[head] = y;
        if (count < HISTORY_MAX) {
            count++;
        }

        // 背景を描画してクリア
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
        SDL_RenderClear(renderer);

        // 足跡の線を描画
        int index = head; // リングバッファの先頭位置から開始
        for (int i = count; i > 1; i--) {
            int prev_index = (index - 1 + HISTORY_MAX) % HISTORY_MAX; // リングバッファの1つ前の位置
            int alpha = (128*i/count); // 透明度を変化させる
            SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_BLEND);
            SDL_SetRenderDrawColor(renderer, 255, 255, 255, alpha);
            SDL_RenderDrawLine(renderer, X[prev_index], Y[prev_index], X[index], Y[index]);
            index = prev_index; // 1つ前の位置に移動
        }

        // 画面を更新
        SDL_RenderPresent(renderer);
        SDL_Delay(16); // 16ミリ秒待機 (約60fps)
    }

    // 終了
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();
}
